import { Component, OnInit, OnDestroy } from '@angular/core';
import { HttpHeaders, HttpResponse } from '@angular/common/http';
import { ActivatedRoute } from '@angular/router';
import { Subscription } from 'rxjs';
import { EventManager } from 'app/core/util/event-manager.service';

import { ParseLinks } from 'app/core/util/parse-links.service';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';

import { ISavedSearches } from 'app/shared/model/saved-searches.model';

import { ITEMS_PER_PAGE } from 'app/config/pagination.constants';
import { SavedSearchesService } from './saved-searches.service';
import { SavedSearchesDeleteDialogComponent } from './saved-searches-delete-dialog.component';

@Component({
  selector: 'jhi-saved-searches',
  templateUrl: './saved-searches.component.html',
})
export class SavedSearchesComponent implements OnInit, OnDestroy {
  savedSearches: ISavedSearches[];
  eventSubscriber?: Subscription;
  itemsPerPage: number;
  links: any;
  page: number;
  predicate: string;
  ascending: boolean;
  currentSearch: string;

  constructor(
    protected savedSearchesService: SavedSearchesService,
    protected eventManager: EventManager,
    protected modalService: NgbModal,
    protected parseLinks: ParseLinks,
    protected activatedRoute: ActivatedRoute
  ) {
    this.savedSearches = [];
    this.itemsPerPage = ITEMS_PER_PAGE;
    this.page = 0;
    this.links = {
      last: 0,
    };
    this.predicate = 'id';
    this.ascending = true;
    this.currentSearch =
      this.activatedRoute.snapshot && this.activatedRoute.snapshot.queryParams['search']
        ? this.activatedRoute.snapshot.queryParams['search']
        : '';
  }

  loadAll(): void {
    if (this.currentSearch) {
      this.savedSearchesService
        .search({
          query: this.currentSearch,
          page: this.page,
          size: this.itemsPerPage,
          sort: this.sort(),
        })
        .subscribe((res: HttpResponse<ISavedSearches[]>) => this.paginateSavedSearches(res.body, res.headers));
      return;
    }

    this.savedSearchesService
      .query({
        page: this.page,
        size: this.itemsPerPage,
        sort: this.sort(),
      })
      .subscribe((res: HttpResponse<ISavedSearches[]>) => this.paginateSavedSearches(res.body, res.headers));
  }

  reset(): void {
    this.page = 0;
    this.savedSearches = [];
    this.loadAll();
  }

  loadPage(page: number): void {
    this.page = page;
    this.loadAll();
  }

  search(query: string): void {
    this.savedSearches = [];
    this.links = {
      last: 0,
    };
    this.page = 0;
    if (query) {
      this.predicate = '_score';
      this.ascending = false;
    } else {
      this.predicate = 'id';
      this.ascending = true;
    }
    this.currentSearch = query;
    this.loadAll();
  }

  ngOnInit(): void {
    this.loadAll();
    this.registerChangeInSavedSearches();
  }

  ngOnDestroy(): void {
    if (this.eventSubscriber) {
      this.eventManager.destroy(this.eventSubscriber);
    }
  }

  trackId(index: number, item: ISavedSearches): number {
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
    return item.id!;
  }

  registerChangeInSavedSearches(): void {
    this.eventSubscriber = this.eventManager.subscribe('savedSearchesListModification', () => this.reset());
  }

  delete(savedSearches: ISavedSearches): void {
    const modalRef = this.modalService.open(SavedSearchesDeleteDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.savedSearches = savedSearches;
  }

  sort(): string[] {
    const result = [this.predicate + ',' + (this.ascending ? 'asc' : 'desc')];
    if (this.predicate !== 'id') {
      result.push('id');
    }
    return result;
  }

  protected paginateSavedSearches(data: ISavedSearches[] | null, headers: HttpHeaders): void {
    const headersLink = headers.get('link');
    this.links = this.parseLinks.parse(headersLink ? headersLink : '');
    if (data) {
      for (let i = 0; i < data.length; i++) {
        this.savedSearches.push(data[i]);
      }
    }
  }
}
